@@ -57,18 +57,14 @@ final class ContextMenu extends MouseAdapter {
         final ResultSetTableModel model = (ResultSetTableModel)table.getModel();
         final int x = e.getX();
         final int y = e.getY();
-        if (table.columnAtPoint(new Point(x, y)) <= 0) {
-            return;
-        }
         final int selectedColumn = table.getSelectedColumn();
         final int selectedRow = table.getSelectedRow();
+        final boolean isUpdatable = model.isUpdatable();
+        final boolean isCellSelected = selectedColumn >= 0;
         final boolean isCellEditable = model.isCellEditable(selectedRow,
                                                             selectedColumn);
         final boolean isSelectedInsertRow = model.isInsertRow(selectedRow);
-        final boolean isClipboardEnabled = Toolkit.getDefaultToolkit()
-                                                  .getSystemClipboard()
-                                                  .getContents(null)
-                                                  .isDataFlavorSupported(DataFlavor.stringFlavor);
+        final boolean isClipboardEnabled = isClipboardEnabled();
         JPopupMenu popup = new JPopupMenu();
         popup.add(createJMenuItem(getMessage("table.copy.cell"),
                                   'C',
@@ -79,7 +75,8 @@ final class ContextMenu extends MouseAdapter {
                                       }
 
                                   },
-                                  getKeyStroke(KeyEvent.VK_C, true)));
+                                  getKeyStroke(KeyEvent.VK_C, true)))
+             .setEnabled(isCellSelected);
         popup.add(createJMenuItem(getMessage("table.paste.cell"),
                                   'P',
                                   new AbstractAction() {
@@ -143,7 +140,9 @@ final class ContextMenu extends MouseAdapter {
 
                                   },
                                   getKeyStroke(KeyEvent.VK_I, true)))
-             .setEnabled(isClipboardEnabled && !isSelectedInsertRow);
+             .setEnabled(isUpdatable
+                         && isClipboardEnabled
+                         && !isSelectedInsertRow);
         popup.add(createJMenuItem(getMessage("table.fix.insert"),
                                   'F',
                                   new AbstractAction() {
@@ -174,13 +173,12 @@ final class ContextMenu extends MouseAdapter {
      */
     private void showMenuForJTableHeader(JTableHeader component, MouseEvent e) {
         final JTable table = component.getTable();
+        final ResultSetTableModel model = (ResultSetTableModel)table.getModel();
         final int x = e.getX();
         final int y = e.getY();
         final int columnIndex = component.columnAtPoint(new Point(x, y));
-        final boolean isClipboardEnabled = Toolkit.getDefaultToolkit()
-                                                  .getSystemClipboard()
-                                                  .getContents(null)
-                                                  .isDataFlavorSupported(DataFlavor.stringFlavor);
+        final boolean isUpdatable = model.isUpdatable();
+        final boolean isClipboardEnabled = isClipboardEnabled();
         JPopupMenu popup = new JPopupMenu();
         popup.add(createJMenuItem(getMessage("header.sort"),
                                   'S',
@@ -211,7 +209,8 @@ final class ContextMenu extends MouseAdapter {
                                           onMenuAddNewRowSelected(table);
                                       }
 
-                                  }));
+                                  }))
+             .setEnabled(isUpdatable);
         popup.add(createJMenuItem(getMessage("table.insert.paste"),
                                   'O',
                                   new AbstractAction() {
@@ -222,7 +221,7 @@ final class ContextMenu extends MouseAdapter {
 
                                   },
                                   getKeyStroke(KeyEvent.VK_I, true)))
-             .setEnabled(isClipboardEnabled);
+             .setEnabled(isUpdatable && isClipboardEnabled);
         popup.show(component, e.getX(), e.getY());
     }
 
@@ -232,6 +231,7 @@ final class ContextMenu extends MouseAdapter {
      * @param e }EXCxg
      */
     private void showMenuForJTextComponent(Component component, MouseEvent e) {
+        boolean isClipboardEnabled = isClipboardEnabled();
         JPopupMenu popup = new JPopupMenu();
         popup.add(createJMenuItem(getMessage("text.cut"),
                                   'T',
@@ -244,7 +244,8 @@ final class ContextMenu extends MouseAdapter {
         popup.add(createJMenuItem(getMessage("text.paste"),
                                   'P',
                                   new DefaultEditorKit.PasteAction(),
-                                  getKeyStroke(KeyEvent.VK_V, true)));
+                                  getKeyStroke(KeyEvent.VK_V, true)))
+             .setEnabled(isClipboardEnabled);
         popup.show(component, e.getX(), e.getY());
     }
 
@@ -295,19 +296,56 @@ final class ContextMenu extends MouseAdapter {
                 : 0));
     }
 
+    /**
+     * Nbv{[hp\ǂ𒲍B
+     * Nbv{[hef[^Ƃ݂Ȃꂽꍇɗp\ƂB
+     * @return Nbv{[hp\Ȃ<code>true</code>AłȂ<code>false</code>
+     */
+    private static boolean isClipboardEnabled() {
+        return Toolkit.getDefaultToolkit()
+                      .getSystemClipboard()
+                      .getContents(null)
+                      .isDataFlavorSupported(DataFlavor.stringFlavor);
+    }
+
     /**
      * j["ZRs["IꂽƂ̏B
      * @param table e[u
      */
     void onMenuCopySelectedCellValueSelected(JTable table) {
-        int selectedColumn = table.getSelectedColumn();
-        int selectedRow = table.getSelectedRow();
-        Object o = table.getValueAt(selectedRow, selectedColumn);
-        if (o != null) {
+        ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        Exporter exporter = new SimpleExporter(bos, "\t");
+        int[] selectedColumns = table.getSelectedColumns();
+        int[] selectedRows = table.getSelectedRows();
+        int columnCount = selectedColumns.length;
+        try {
+            for (int y = 0; y < selectedRows.length; y++) {
+                int rowIndex = selectedRows[y];
+                Object[] values = new Object[columnCount];
+                for (int x = 0; x < columnCount; x++) {
+                    int columnIndex = selectedColumns[x];
+                    Object value = table.getValueAt(rowIndex, columnIndex);
+                    if (value == null) {
+                        value = "";
+                    }
+                    values[x] = value;
+                }
+                exporter.addRow(values);
+            }
+            String contents = bos.toString().replaceAll("\\r", "");
+            if (log.isTraceEnabled()) {
+                Writer buffer = new StringWriter();
+                PrintWriter out = new PrintWriter(buffer);
+                out.println("to clipboard:");
+                out.print(contents);
+                log.trace(buffer);
+            }
             Clipboard clipboard = Toolkit.getDefaultToolkit()
                                          .getSystemClipboard();
-            StringSelection ss = new StringSelection(o.toString());
+            StringSelection ss = new StringSelection(contents);
             clipboard.setContents(ss, ss);
+        } catch (IOException ex) {
+            onError(ex);
         }
     }
 
@@ -317,19 +355,10 @@ final class ContextMenu extends MouseAdapter {
      */
     void onMenuPasteValueToSelectedCellSelected(JTable table) {
         Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
+        Transferable t = clipboard.getContents(null);
         try {
-            Transferable t = clipboard.getContents(null);
-            int selectedColumn = table.getSelectedColumn();
-            int selectedRow = table.getSelectedRow();
-            Object oldValue = table.getValueAt(selectedRow, selectedColumn);
-            Object newValue = t.getTransferData(DataFlavor.stringFlavor);
-            ResultSetTableModel model = (ResultSetTableModel)table.getModel();
-            model.updateValueChanged(table,
-                                     selectedRow,
-                                     selectedColumn,
-                                     oldValue,
-                                     newValue);
-            table.setValueAt(newValue, selectedRow, selectedColumn);
+            String contents = (String)t.getTransferData(DataFlavor.stringFlavor);
+            paste(table, contents);
         } catch (UnsupportedFlavorException ex) {
             onError(ex);
         } catch (IOException ex) {
@@ -339,6 +368,44 @@ final class ContextMenu extends MouseAdapter {
         }
     }
 
+    /**
+     * \tsB
+     * @param table e[u
+     * @param contents \te
+     * @throws IOException o̓G[ꍇ
+     * @throws SQLException SQL֘AG[ꍇ
+     */
+    private void paste(JTable table, String contents) throws IOException, SQLException {
+        ResultSetTableModel model = (ResultSetTableModel)table.getModel();
+        Importer importer = new TabSeparatedTextImporter(contents);
+        try {
+            int[] selectedColumns = table.getSelectedColumns();
+            int[] selectedRows = table.getSelectedRows();
+            int columnCount = selectedColumns.length;
+            for (int y = 0; y < selectedRows.length; y++) {
+                int rowIndex = selectedRows[y];
+                boolean isInsertRow = model.isInsertRow(rowIndex);
+                Object[] values = importer.nextRow();
+                int limit = Math.min(columnCount, values.length);
+                for (int x = 0; x < limit; x++) {
+                    int columnIndex = selectedColumns[x];
+                    Object oldValue = table.getValueAt(rowIndex, columnIndex);
+                    Object newValue = values[x];
+                    if (!isInsertRow) {
+                        model.updateValueChanged(table,
+                                                 rowIndex,
+                                                 columnIndex,
+                                                 oldValue,
+                                                 newValue);
+                    }
+                    table.setValueAt(newValue, rowIndex, columnIndex);
+                }
+            }
+        } finally {
+            importer.close();
+        }
+    }
+
     /**
      * j["ׂđI"IꂽƂ̏B
      * @param table e[u
@@ -348,21 +415,28 @@ final class ContextMenu extends MouseAdapter {
     }
 
     /**
-     * j["l̃NA"IꂽƂ̏B
+     * j["Null̐ݒ"IꂽƂ̏B
      * @param table e[u
      */
     void onMenuResetValueSelected(JTable table) {
         try {
-            int selectedColumn = table.getSelectedColumn();
-            int selectedRow = table.getSelectedRow();
-            Object oldValue = table.getValueAt(selectedRow, selectedColumn);
             ResultSetTableModel model = (ResultSetTableModel)table.getModel();
-            model.updateValueChanged(table,
-                                     selectedRow,
-                                     selectedColumn,
-                                     oldValue,
-                                     null);
-            table.setValueAt(null, selectedRow, selectedColumn);
+            int[] selectedColumns = table.getSelectedColumns();
+            int[] selectedRows = table.getSelectedRows();
+            int columnCount = selectedColumns.length;
+            for (int y = 0; y < selectedRows.length; y++) {
+                int rowIndex = selectedRows[y];
+                for (int x = 0; x < columnCount; x++) {
+                    int columnIndex = selectedColumns[x];
+                    Object oldValue = table.getValueAt(rowIndex, columnIndex);
+                    model.updateValueChanged(table,
+                                             rowIndex,
+                                             columnIndex,
+                                             oldValue,
+                                             null);
+                    table.setValueAt(null, rowIndex, columnIndex);
+                }
+            }
         } catch (SQLException ex) {
             onError(ex);
         }
@@ -385,11 +459,12 @@ final class ContextMenu extends MouseAdapter {
      */
     void onMenuCopyInsertRowSelected(JTable table) {
         ResultSetTableModel model = (ResultSetTableModel)table.getModel();
-        int rowCount = model.getRowCount();
-        int selectedRow = table.getSelectedRow();
-        List row = (List)model.getDataVector().get(selectedRow);
-        model.addRowAsInsert(row.toArray());
-        table.changeSelection(rowCount, 1, false, false);
+        int[] selectedRows = table.getSelectedRows();
+        for (int i = 0; i < selectedRows.length; i++) {
+            List row = (List)model.getDataVector().get(selectedRows[i]);
+            model.addRowAsInsert(row.toArray());
+            table.changeSelection(model.getRowCount(), 1, false, false);
+        }
     }
 
     /**
@@ -445,7 +520,10 @@ final class ContextMenu extends MouseAdapter {
     void onMenuFixInsertSelected(JTable table) {
         try {
             ResultSetTableModel model = (ResultSetTableModel)table.getModel();
-            model.updateRowInserted(table);
+            int[] selectedRows = table.getSelectedRows();
+            for (int i = 0; i < selectedRows.length; i++) {
+                model.updateRowInserted(table, selectedRows[i]);
+            }
         } catch (SQLException ex) {
             onError(ex);
         } catch (RuntimeException ex) {
@@ -459,16 +537,23 @@ final class ContextMenu extends MouseAdapter {
      */
     void onMenuDeleteRowSelected(JTable table) {
         ResultSetTableModel model = (ResultSetTableModel)table.getModel();
-        int selectedRow = table.getSelectedRow();
-        boolean isSelectedInsertRow = model.isInsertRow(selectedRow);
-        if (!isSelectedInsertRow) {
-            try {
-                model.updateRowDeleted(table);
-            } catch (SQLException ex) {
-                onError(ex);
+        int[] selectedRows = table.getSelectedRows();
+        int selectedRowCount = selectedRows.length;
+        if (selectedRowCount < 1) {
+            return;
+        }
+        int selectedRow = selectedRows[0];
+        for (int i = 0; i < selectedRowCount; i++) {
+            boolean isSelectedInsertRow = model.isInsertRow(selectedRow);
+            if (!isSelectedInsertRow) {
+                try {
+                    model.updateRowDeleted(table, new int[]{selectedRow});
+                } catch (SQLException ex) {
+                    onError(ex);
+                }
             }
+            model.removeRow(selectedRow);
         }
-        model.removeRow(selectedRow);
     }
 
     /**
@@ -491,7 +576,7 @@ final class ContextMenu extends MouseAdapter {
     void onMenuSelectColumnSelected(JTable table, int columnIndex) {
         if (columnIndex >= 0) {
             ResultSetTable t = (ResultSetTable)table;
-            t.selectColumn(columnIndex);
+            t.selectColumn(columnIndex, columnIndex);
         }
     }
 
