@@ -28,6 +28,8 @@ class ResultSetTable extends JTable {
     private int limitCount;
     private boolean[] sortedAscArray;
     private String columnAdjustMode;
+    private int dragStartColumn;
+    private int dragStartRow;
 
     /**
      * ResultSetTable̐B
@@ -66,7 +68,7 @@ class ResultSetTable extends JTable {
     }
 
     /**
-     * 񕝒[h̐ݒ
+     * 񕝒[h̐ݒB
      * @param columnAdjustMode 񕝒[h
      */
     void setColumnAdjustMode(String columnAdjustMode) {
@@ -82,9 +84,19 @@ class ResultSetTable extends JTable {
         addMouseListener(new MouseAdapter() {
 
             /* (overridden)
-             * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
+             * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
              */
-            public void mouseClicked(MouseEvent e) {
+            public void mousePressed(MouseEvent e) {
+                onTableClicked(e);
+            }
+
+        });
+        addMouseMotionListener(new MouseMotionAdapter() {
+
+            /* (overridden)
+             * @see java.awt.event.MouseMotionAdapter#mouseDragged(java.awt.event.MouseEvent)
+             */
+            public void mouseDragged(MouseEvent e) {
                 onTableClicked(e);
             }
 
@@ -95,9 +107,19 @@ class ResultSetTable extends JTable {
         header.addMouseListener(new MouseAdapter() {
 
             /* (overridden)
-             * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
+             * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
              */
-            public void mouseClicked(MouseEvent e) {
+            public void mousePressed(MouseEvent e) {
+                onTableHeaderClicked(e);
+            }
+
+        });
+        header.addMouseMotionListener(new MouseMotionAdapter() {
+
+            /* (overridden)
+             * @see java.awt.event.MouseMotionAdapter#mouseDragged(java.awt.event.MouseEvent)
+             */
+            public void mouseDragged(MouseEvent e) {
                 onTableHeaderClicked(e);
             }
 
@@ -225,22 +247,55 @@ class ResultSetTable extends JTable {
      * @param e }EXCxg
      */
     final void onTableHeaderClicked(MouseEvent e) {
-        int x = e.getX();
-        int y = e.getY();
-        Point p = new Point(x, y);
+        if (getRowCount() == 0) {
+            return;
+        }
+        Point p = new Point(e.getX(), e.getY());
         if (SwingUtilities.isLeftMouseButton(e)) {
-            selectColumn(columnAtPoint(p));
+            int id = e.getID();
+            boolean isMousePressed = id == MouseEvent.MOUSE_PRESSED;
+            boolean isMouseDragged = id == MouseEvent.MOUSE_DRAGGED;
+            if (isMousePressed || isMouseDragged) {
+                clearSelection();
+                int columnIndex = columnAtPoint(p);
+                if (columnIndex < 0 || getColumnCount() <= columnIndex) {
+                    return;
+                }
+                int index0;
+                int index1;
+                if (isMousePressed) {
+                    dragStartColumn = columnIndex;
+                    index0 = columnIndex;
+                    index1 = columnIndex;
+                } else if (isMouseDragged) {
+                    index0 = dragStartColumn;
+                    index1 = columnIndex;
+                } else {
+                    return;
+                }
+                selectColumn(index0, index1);
+            }
         }
     }
 
     /**
      * IB
      * @param columnIndex CfbNX
+     * @deprecated
      */
     void selectColumn(int columnIndex) {
-        if (columnIndex >= 0 && getRowCount() > 0) {
+        selectColumn(columnIndex, columnIndex);
+    }
+
+    /**
+     * IB
+     * @param index0 JnCfbNX
+     * @param index1 ICfbNX
+     */
+    void selectColumn(int index0, int index1) {
+        if (getRowCount() > 0) {
             clearSelection();
-            addColumnSelectionInterval(columnIndex, columnIndex);
+            addColumnSelectionInterval(index0, index1);
             addRowSelectionInterval(getRowCount() - 1, 0);
         }
     }
@@ -250,19 +305,33 @@ class ResultSetTable extends JTable {
      * @param e }EXCxg
      */
     final void onTableClicked(MouseEvent e) {
-        int x = e.getX();
-        int y = e.getY();
-        Point p = new Point(x, y);
+        Point p = new Point(e.getX(), e.getY());
         if (SwingUtilities.isLeftMouseButton(e)) {
             int columnIndex = columnAtPoint(p);
-            if (columnIndex == 0) {
+            int id = e.getID();
+            boolean isMousePressed = id == MouseEvent.MOUSE_PRESSED;
+            boolean isMouseDragged = id == MouseEvent.MOUSE_DRAGGED;
+            if (columnIndex == 0 && (isMousePressed || isMouseDragged)) {
                 clearSelection();
                 int rowIndex = rowAtPoint(p);
-                addRowSelectionInterval(rowIndex, rowIndex);
+                if (rowIndex < 0 || getRowCount() < rowIndex) {
+                    return;
+                }
+                int index0;
+                int index1;
+                if (isMousePressed) {
+                    dragStartRow = rowIndex;
+                    index0 = rowIndex;
+                    index1 = rowIndex;
+                } else if (isMouseDragged) {
+                    index0 = dragStartRow;
+                    index1 = rowIndex;
+                } else {
+                    return;
+                }
+                addRowSelectionInterval(index0, index1);
                 addColumnSelectionInterval(getColumnCount() - 1, 1);
             }
-        } else if (SwingUtilities.isRightMouseButton(e)) {
-            changeSelection(rowAtPoint(p), columnAtPoint(p), false, false);
         }
     }
 
